import 'package:domain/entity/facility_page_entity.dart';
import 'package:domain/repository/facility_repository.dart';
import 'package:plugin_platform/http/dio/dio_cancelable_mixin.dart';
import 'package:riverpod_annotation/riverpod_annotation.dart';
import 'package:widgets/load_state_layout.dart';
import 'package:widgets/widget_export.dart';

import 'facility_deposit_state.dart';

part 'facility_deposit_view_model.g.dart';

@riverpod
class FacilityDepositViewModel extends _$FacilityDepositViewModel with DioCancelableMixin{
  late final FacilityRepository _facilityRepository;
  var _curPage = 1; //请求参数当前的页面
  var _needShowPlaceholder = true; //是否展示LoadingView

  // Refresh 控制器
  final EasyRefreshController refreshController = EasyRefreshController(
    controlFinishRefresh: true,  //允许刷新
    controlFinishLoad: false,   //不允许加载
  );

  @override
  FacilityDepositState build() {
    _facilityRepository = ref.read(facilityRepositoryProvider);
    final state = FacilityDepositState(datas: []);
    registerCancellation();
    return state;
  }

  //刷新页面状态
  void changeLoadingState(LoadState loadState, String? errorMsg) {
    state = state.copyWith(loadingState: loadState, errorMessage: errorMsg);
  }

  // Refresh 刷新事件
  Future onRefresh() async {
    _curPage = 1;
    fetchList();
  }

  // 重试请求
  Future retryRequest() async {
    _curPage = 1;
    _needShowPlaceholder = true;
    fetchList();
  }

  /// 获取服务器数据
  Future fetchList() async {
    if (_needShowPlaceholder) {
      changeLoadingState(LoadState.State_Loading, null);
    }

    // 获取列表
    final result = await _facilityRepository.fetchFacilityDepositList(
      curPage: _curPage,
      cancelToken: cancelToken,
    );

    // 处理数据
    if (result.isSuccess) {
      state = state.copyWith(totalDeposit: result.data?.totalDeposit);
      handleList(result.data?.list);
    } else {
      changeLoadingState(LoadState.State_Error, result.errorMsg);
    }

    // 最后赋值
    _needShowPlaceholder = false;
  }

  // 处理数据与展示的逻辑
  void handleList(List<FacilityPageList>? list) {
    if (list != null && list.isNotEmpty) {
      //有数据，判断是刷新还是加载更多的数据
      if (_curPage == 1) {
        //刷新的方式
        state = state.copyWith(datas: list);
        refreshController.finishRefresh();

        //更新展示的状态
        changeLoadingState(LoadState.State_Success, null);
      } else {
        //加载更多
        state.datas.addAll(List<FacilityPageList>.from(state.datas)..addAll(list));
        refreshController.finishLoad();
      }
    } else {
      if (_curPage == 1) {
        //展示无数据的布局
        state = state.copyWith(datas: []);
        changeLoadingState(LoadState.State_Empty, null);
        refreshController.finishRefresh();
      } else {
        //展示加载完成，没有更多数据了
        refreshController.finishLoad(IndicatorResult.noMore);
      }
    }
  }

}
